#! /usr/uumath/bin/bash -
### -*-sh-*-
### ====================================================================
###  @UNIX-shell-file{
###     author          = "Nelson H. F. Beebe",
###     version         = "0.11",
###     date            = "08 June 2017",
###     time            = "07:20:40 MST",
###     filename        = "cattobib.sin",
###     address         = "University of Utah
###                        Department of Mathematics, 110 LCB
###                        155 S 1400 E RM 233
###                        Salt Lake City, UT 84112-0090
###                        USA",
###     telephone       = "+1 801 581 5254",
###     FAX             = "+1 801 581 4148",
###     URL             = "http://www.math.utah.edu/~beebe",
###     checksum        = "32035 964 3964 35195",
###     email           = "beebe@math.utah.edu, beebe@acm.org,
###                        beebe@computer.org (Internet)",
###     codetable       = "ISO/ASCII",
###     keywords        = "BibTeX; SUTRS library catalog records;
###                        USMARC library catalog records; Z39.50
###                        library catalog server",
###     license         = "public domain",
###     supported       = "yes",
###     docstring       = "Search the Library of Congress catalog via
###                        its Z39.50 interface for one or more
###                        publications by author, title, ISBN, or ISSN,
###                        and convert the US MARC records found to
###                        BibTeX on stdout.  It is NOT necessary to
###                        remove hyphens from ISBN or ISSN data.
###
###                        Usage:
###                       	cattobib [ --? ]			\
###                       		 [ --author ]			\
###                       		 [ --byxxx ]			\
###                       		 [ --CODEN ]			\
###                       		 [ --debug ]			\
###                       		 [ --editor ]			\
###                       		 [ --help ]			\
###                       		 [ --initfile alt-cattobibrc-file ] \
###                       		 [ --ISBN ]			\
###                       		 [ --ISSN ]			\
###                       		 [ --ISSN-L ]			\
###                       		 [ --keep-files ]		\
###                       		 [ --LCCN ]			\
###                       		 [ --logfile new-filename ]	\
###					 [ --max number ]		\
###                       		 [ --no-USMARC ]		\
###                       		 [ --publisher ]		\
###                       		 [ --quiet ]			\
###                       		 [ --series ]			\
###                       		 [ --server server-address-or-alias ] \
###                       		 [ --test ]			\
###                       		 [ --title ]			\
###                       		 [ --USMARC ]			\
###                       		 [ --version ]			\
###                       		 [ --volume ]			\
###                       		 [ --year ]			\
###                       		 search-key(s) ... > BibTeX-file
###
###                        Options may be abbreviated to any unique
###                        leading prefix, and may begin with either one
###                        or two hyphens.  Any option beginning -by...
###                        is assumed to be a bibsort option, and is
###                        passed verbatim to it.
###
###                        For --volume, the search keys must contain a
###                        series name and volume number, separated by
###                        one or more nonalphanumeric, nonhyphen,
###                        nonspace delimiters, such as
###
###                       	"Lecture notes in computer science / 1234"
###                       	"Lecture notes in computer science :: 2345"
###                       	"Lecture notes in computer science ;;; 3455"
###                       	"Lecture notes in computer science ==== 4567"
###
###			   [08-Jun-2017] -- Version 0.11 with minor tweaks in
###					    author, ISBN, and pages data
###					    conversion.
###
###			   [09-Jan-2015] -- version 0.10 with minor tweaks in
###			   		    volume and ISBN data conversion.
###
###			   [10-Nov-2014] -- version 0.09 with changes
###					    to quoting of carets, and tweaks
###					    in ISBN and pages data conversion.
###
###			   [29-Apr-2014] -- version 0.08 with
###					    corrections to a few
###					    command-line
###					    abbreviations, addition of
###					    --LCCN, --max,
###					    --publisher, and --year
###					    support, and parsing of
###					    Z39.50 output to finally
###					    handle result count
###					    properly.
###
###			   [14-Sep-2013] -- version 0.07 with minor
###					    tweaks to field handling,
###					    addition of parallel
###					    searching, and --NO-USMARC
###					    and --USMARC options
###
###			   [18-Jun-2011] -- version 0.06 with ISSN-L
###					    support, and many more
###					    built-in server names
###
###			   [26-Nov-2009] -- version 0.05 with user and
###					    local alias support
###
###			   [05-Apr-2006] -- original version
###
###                        The checksum field above contains a CRC-16
###                        checksum as the first value, followed by the
###                        equivalent of the standard UNIX wc (word
###                        count) utility output of lines, words, and
###                        characters.  This is produced by Robert
###                        Solovay's checksum utility.",
###  }
### ====================================================================

IFS='
 	'
umask 077


### Root of local installation tree
prefix=/usr/uumath
bindir=$prefix/bin

TMPDIR=${TMPDIR-/tmp}

### Paths and directories:
PROGRAM=cattobib
VERSION=0.14
SHRLIBDIR=${TESTSHRLIBDIR-$prefix/share/$PROGRAM/${PROGRAM}-$VERSION}

### Programs needed:
AWK=/usr/uumath/bin/nawk
BIBCLEAN=/usr/uumath/bin/bibclean
BIBLABEL=/usr/uumath/bin/biblabel
BIBORDER=/usr/uumath/bin/biborder
BIBORDERFLAGS=--isbn-13
BIBSORT=/usr/uumath/bin/bibsort
CAT=/usr/uumath/bin/cat
CITESUB=/usr/uumath/bin/citesub
DATE=/usr/uumath/bin/date
EXPECT=/usr/bin/expect
EXPR=/usr/uumath/bin/expr
HOSTNAME=/usr/uumath/bin/hostname
LOGGER=/usr/uumath/bin/tee
OD=/usr/uumath/bin/od
SED=/usr/uumath/bin/sed
TEE=/usr/uumath/bin/tee
TEST=/usr/uumath/bin/test
YAZ_CLIENT=/usr/uumath/bin/yaz-client
Z39_50_TO_BIBTEX=$SHRLIBDIR/usmarc-to-bibtex.awk

### Random suffix to prevent collisions in $TMPDIR, and discourage
### guessing attacks.
SFX=$$
SFX=$SFX.`$DATE +%N`
test -n "$RANDOM"    && SFX=$SFX.$RANDOM
test -r /dev/urandom && SFX="$SFX.`dd ibs=8 count=1 < /dev/urandom 2> /dev/null | $OD -t x8 | awk 'NF == 2 {print $2}'`"

### Files, flags and variables:
BIBCLEANFLAGS="-max-width 0"
BIBCLEANFLAGS=
BIBSORTFLAGS=
COMBINEDRC=$TMPDIR/$PROGRAM.rc.$SFX
CATTOBIBRC=$HOME/.cattobibrc
DEBUG=0
DEFAULT_Z39_50_SERVER="z3950.loc.gov:7090/Voyager"
EXTRA=
FIFO=$TMPDIR/$PROGRAM.fifo.$SFX
flags=
FORMAT="USMARC"
LABELFILE=$TMPDIR/$PROGRAM.label.$SFX
LOGFILE=$TMPDIR/$PROGRAM.$HOSTNAME.`$DATE +%Y.%m.%d.%H.%M.%S`
LOGFILE=/dev/null
MAX=10
PACKAGE_DATE="25-May-2019"
PACKAGE_VERSION="0.14"
SEDINPUTFILTER=$CAT
SEDOUTPUTFILTER=$CAT
SERIESVOLUME=
SHRLIBRC=$SHRLIBDIR/cattobib.rc
TEST_MODE=no
TMPBIBTEXFILE=$TMPDIR/$PROGRAM.bib.$SFX
USMARC=yes
YEAR=${YEAR-}

# Z39.50 catalog servers:
Z39_50_SERVER="$DEFAULT_Z39_50_SERVER"
Z39_50_SERVER_LIST=""

# Ensure consistent sorting across all platforms
LC_ALL=C
export LC_ALL

trap "rm -f $COMBINEDRC $FIFO $LABELFILE $TMPBIBTEXFILE"  EXIT HUP INT QUIT PIPE TERM

error_and_exit()
{
    echo "ERROR: ""$@" 1>&2
    exit 1
}

usage()
{
    cat <<EOF
$0 [--?]
		    [ --author ]
		    [ --byxxx ]
		    [ --CODEN ]
		    [ --debug ]
		    [ --editor ]
		    [ --help ]
		    [ --initfile alt-cattobibrc-file ]
		    [ --ISBN ]
		    [ --ISSN ]
		    [ --ISSN-L ]
		    [ --keep-files ]
		    [ --LCCN ]
		    [ --logfile new-filename ]
		    [ --max number ]
		    [ --no-USMARC ]
		    [ --publisher ]
		    [ --quiet ]
		    [ --series ]
		    [ --server server-address-or-alias-list ]
		    [ --test ]
		    [ --title ]
		    [ --USMARC ]
		    [ --version ]
		    [ --volume ]
		    [ --year year ]
		    search-key-1 search-key-2 search-key-3 ... >BibTeX-file

Search keys should be either ISBNs, or quoted title strings, unless
one of the qualifier options --author, --CODEN, --editor, --ISBN,
--ISSN, --ISSN-L, or --title is specified, in which case, the search
strings should be of that type.

For --volume, the search keys must contain a series name and volume
number, separated by one or more nonalphanumeric, nonhyphen, nonspace
delimiters, such as

	"Lecture notes in computer science / 1234"
	"Lecture notes in computer science :: 2345"
	"Lecture notes in computer science ;;; 3455"
	"Lecture notes in computer science ==== 4567"

WARNING: Some Z39.50 servers are unable to handle the compound queries
needed for the --volume option.

All searches are completed before the data is processed further to
produce BibTeX entries, so if you interrupt execution, there will
likely be no useful output.

The server can be supplied as an explicit Z39.50 address, such as

			 $DEFAULT_Z39_50_SERVER

(the default server at the US Library of Congress), or more usually,
by a shorthand alias name (vertical bars separate alternatives, and
asterisk matches any word with that prefix):

	ALL | A*			All Z39.50 servers known to cattobib
	NATIONAL | N*			National libraries and union catalogs
	alberta | ab			University of Alberta
	amherst | umass			University of Massachusetts, Amherst
	amicus | ca			National Library of Canada
	anu				Australian National University
	be | ulb			Universit Libre de Bruxelles, Belgium
	bibsys              		Norwegian Union Catalog
	bne | es			National Library of Spain
	bnp | pt            		National Library of Portugal
	boulder | co			University of Colorado, Boulder
	british | br			British Library
	byu				Brigham Young University
	calgary				University of Calgary
	caltech				California Institute of Technology
	carnegie | cmu			Carnegie Mellon University
	chicago				University of Chicago
	columbia | cu			Columbia University
	congress | lc | loc		US Library of Congress
	copac | uk			COPAC (union of 24 research-university
					catalogs in the UK and Ireland)
	cosmos				Danish National Library of Science & Medicine
	crl				Center for Research Libraries
	dartmouth			Dartmouth College, NH
	denmark | dk			Royal Library of Denmark
	dsb | dsl			Danish State Library
	duke				Duke University
	edinburgh | ed			Edinburgh University
	emory				Emory University
	eu				European University Institute Library
	florida | fl			Florida Center for Library Automation
	gbv				German Union Catalog
	gmu				George Mason University
	harvard				Harvard University
	hopkins | jhu			The Johns Hopkins University
	indiana | iu			Indiana University
	indystate | in			Indiana State University Consortium
	kings | kcl			King's College (London)
	ku-leuven | ku			Katholieke Universiteit Leuven, Belgium
	madrid				Universidad Autonoma de Madrid, Spain
	marriott | utah | ut		University of Utah Marriott Library
	mcgill				McGill University
	melbourne			University of Melbourne
	melvyl | cal			University of California
	michigan | mi			University of Michigan
	minn* | mn			University of Minnesota
	mit				Massachusetts Institute of Technology
	montreal | um			Universit de Montral
	nbi | bohr			Niels Bohr Institut
	newyork | ny			New York University
	nla | au			National Library of Australia
	nlm | nih			National Library of Medicine (US)
	nlnz | nz			National Library of New Zealand
	nls | scotland			National Library of Scotland
	nlw | cwm | wales		National Library of Wales
	northwestern | nw		Northwestern University
	norway | no			National Library of Norway
	nsw | unsw			University of New South Wales
	nus | sg			National University of Singapore
	odense | sdu			University of Southern Denmark
	oregon | uo			University of Oregon
	oxford | ox*			Oxford University
	pennstate | psu | pa		Pennsylvania State University
	poland | pl			National Library of Poland
	princeton			Princeton University
	quebec | uq			Universit de Qubec
	rlg				Research Libraries Group
	rutgers				Rutgers University
	sfu				Simon Fraser University
	sudoc | abes | fr		French Union Catalog
	sweden | se			National Library of Sweden
	stanford | su			Stanford University
	stockholm			Stockholm University
	tamu				Texas A&M University
	texas | tx			University of Texas at Austin
	toronto				University of Toronto
	trinity | tcd			Trinity College, Dublin
	tub | berlin			Technische Universitt Berlin, Germany
	tud | darmstadt			Technische Universitt Darmstadt, Germany
	tufts				Tufts University
	ub | bern			Universitt Bern, Switzerland
	ucc | cork			University College Cork, Ireland
	ucd | dublin			University College Dublin, Ireland
	ucsf				University of California, San Francisco
	upenn | penn			University of Pennsylvania
	usc				University of Southern California
	uta				University of Texas, Arlington
	uwo				University of Western Ontario
	vanderbilt			Vanderbilt University
	victoria | vu			Victoria University, Melbourne, Australia
	westpoint | usma		United States Military Academy West Point
	witwatersrand | wit		University of Witwatersrand, Johannesburg, South Africa
	wustl				Washington University St. Louis
	yale				Yale University

Please note that data from ANY of those libraries are likely to be error
ridden: it is best to repeat your searches with three or more servers in
order to improve data quality and reliability.

EOF

    aliasfiles=""

    test -f $CATTOBIBRC && aliasfiles=" $aliasfiles $CATTOBIBRC"

    test -f $SHRLIBRC && aliasfiles=" $aliasfiles $SHRLIBRC"

    test -n "$aliasfiles" &&
	echo "Additional server aliases and Z39.50 paths are available here:" &&
	    (echo "$aliasfiles" | $SED -e 's/  */\n\t/g') &&
		echo

    cat <<EOF
Further user-specific and system-specific customization is possible:

	--server home	selects ALL of the user-defined Z39.50 settings in
			$CATTOBIBRC
			[if it exists]

	--server local	selects ALL of the locally-defined Z39.50 settings in
			$SHRLIBRC
			[if it exists]
EOF
}

usage_and_exit()
{
    usage
    exit 0
}

version_and_exit()
{
    echo "This is `basename $0` version $PACKAGE_VERSION [$PACKAGE_DATE]"
    exit 0
}

### ====================================================================
### NB: Z39.50 "@attr n=m" settings for our flags assignments are
### documented in, e.g.,
###
###	http://www.loc.gov/z3950/agency/defns/bib1.html
###	http://support.epnet.com/custsupport/UserDocumentation/PrimaryDatabases.doc
###	http://www.blueangeltech.com/standards/GeoProfile/annex_a.htm
###
### For example, to do a search on both series and volume within the
### series (e.g., LNCS vol. 3581):
###
###	find @and @attr 1=1066 "3581" @attr 1=5 "Lecture notes in computer science"
###
### That can be done with a command like this:
###
###	cattobib --volume 3851 --series "Lecture notes in computer science"
###
### ====================================================================

while test $# -gt 0
do
	case $1 in

	'--?' | '-?' )
		usage_and_exit
		;;

	 -author |  -autho |  -auth |  -aut |  -au |  -a | \
	--author | --autho | --auth | --aut | --au | --a )
		flags="@attr 1=1003"
		flags="@attr 1=1"
		;;

	--by* | -by*)
		shift
		BIBSORT=$bindir/bibsort
		BIBSORTFLAGS="$BIBSORTFLAGS $1"
		;;

	 -CODEN |  -CODE |  -COD |  -CO |  -C | \
	 -coden |  -code |  -cod |  -co |  -c | \
	--CODEN | --CODE | --COD | --CO | --C | \
	--coden | --code | --cod | --co | --c )
		flags="@attr 1=60"
		;;

	 -debug |  -debu |  -deb |  -de |  -d | \
	--debug | --debu | --deb | --de | --d )
		DEBUG=1
		;;

	 -editor |  -edito |  -edit |  -edi |  -ed |  -e | \
	--editor | --edito | --edit | --edi | --ed | --e )
		flags="@attr 1=1020"
		;;

	 -help |  -hel |  -he |  -h | \
	--help | --hel | --he | --h )
		usage_and_exit
		;;

	 -initfile |  -initfil |  -initfi |  -initf |  -init |  -ini |  -in | \
	--initfile | --initfil | --initfi | --initf | --init | --ini | --in )
		shift
		test ! -r "$1" && echo Initialization file "$1" is not readable: using default file instead
		test   -r "$1" && CATTOBIBRC=$1
		;;

	 -ISBN |  -ISB |  -iSBN |  -iSB |  -iS |  -i | \
	 -isbn |  -isb |  -isbn |  -isb |  -is |  -i | \
	--ISBN | --ISB | --iSBN | --iSB | --iS | --i | \
	--isbn | --isb | --isbn | --isb | --is | --i )
		flags="@attr 1=7"
		;;

	 -ISSN |  -ISS |  -ISSN |  -ISS | \
	 -issn |  -iss |  -issn |  -iss | \
	--ISSN | --ISS | --ISSN | --ISS | \
	--issn | --iss | --issn | --iss )
		flags="@attr 1=8"
		;;

	 -ISSN-L | \
	 -issn-l | \
	--ISSN-L | \
	--issn-l )
		flags="@attr 1=8"
		;;

	 -keep-files |  -keep-file |  -keep-fil |  -keep-fi |  -keep-f |  -keep- | \
	 -keep |  -kee |  -ke |  -k                                              | \
	--keep-files | --keep-file | --keep-fil | --keep-fi | --keep-f | --keep- | \
	--keep | --kee | --ke | --k                                              )
		trap '' EXIT HUP INT QUIT PIPE TERM
		;;

	 -LCCN |  -LCC |  -LC | \
	 -lccn |  -lcc |  -lc | \
	--LCCN | --LCC | --LC | \
	--lccn | --lcc | --lc )
		flags="@attr 1=16"
		;;

	 -log |  -lo |  -l | \
	--log | --lo | --l )
		shift
		$TEST -f "$LOGFILE" && error_and_exit "Log file $LOGFILE already exists"
		;;

	 -max |  -ma |  -m | \
	--max | --ma | --m )
		shift
		MAX=`$EXPR 0 + "$MAX" || exit 1`
		;;

	 -no-USMARC |  -no-USMAR |  -no-USMA |  -no-USM |  -no-US |  -no-U |  -no- |  -n | \
	 -no-usmarc |  -no-usmar |  -no-usma |  -no-usm |  -no-us |  -no-u               | \
	--no-USMARC | --no-USMAR | --no-USMA | --no-USM | --no-US | --no-U | --no- | --n | \
	--no-usmarc | --no-usmar | --no-usma | --no-usm | --no-us | --no-u               )
		USMARC=no
		;;

	 -publisher |  -publishe |  -publish |  -publis |  -publi |  -publ |  -pub |  -pu |  -p | \
	--publisher | --publishe | --publish | --publis | --publi | --publ | --pub | --pu | --p )
		flags="@attr 1=1018"
		;;

	 -quiet |  -quie |  -qui |  -qu |  -q | \
	--quiet | --quie | --qui | --qu | --q )
		LOGGER=$CAT
		LOGFILE=
		;;

	 -series |  -serie |  -seri | \
	--series | --serie | --seri )
		flags="@attr 1=5"
		;;


	 -server |  -serve |  -serv |  -ser |  -se |  -s | \
	--server | --serve | --serv | --ser | --se | --s )
		shift
		## See http://www.indexdata.dk/targettest/u.php for a long list
		## See http://www.bs.dk/danzig for pointers to a list of Danish servers
		## See http://www.nla.gov.au/apps/libraries?action=ListTargets for Australian servers

		for z in $1
		do
			case $z in

			### TO DO: National Library of Portugal is SUTRS
			### format, and not properly supported by ALL and
			### NATIONAL server requests.

			### These work
			ALL | A* )				Z39_50_SERVER="dra.library.ualberta.ca:210/Marion
										umlibr.library.umass.edu:210/INNOPAC
										amicus.nlc-bnc.ca:210/NL
										z3950.bibsys.no:2100/BIBSYS
										www.bne.es:2210/BIMO
										malvine.bn.pt:2100
										libraries.colorado.edu:210/INNOPAC
										z3950cat.bl.uk:9909/BLAC
										clavis.ucalgary.ca:2200/UNICORN
										clas.caltech.edu:210/INNOPAC
										webcat.library.cmu.edu:210/unicorn
										clio-db.cc.columbia.edu:7090/Voyager
										z3950.loc.gov:7090/Voyager
										z3950.copac.ac.uk:2020
										cosmos.dnlb.dk:9909/Aleph
										catalog.crl.edu:210/INNOPAC
										z3950.kb.dk:2100/KGL01
										zeus.statsbiblioteket.dk:210/Horizon
										ducatalog.lib.duke.edu:210/MARION
										z3950.fcla.edu:210/AM
										z3950.gbv.de:20011/gvk
										catalog.lib.jhu.edu:210/horizon
										es33.uits.indiana.edu:2200/unicorn
										libaix.indstate.edu:210/LUIS
										library.kcl.ac.uk:9990/KCL01
										cos.lib.utah.edu:6060
										aleph.mcgill.ca:210/MUSE
										library.unimelb.edu.au:210/INNOPAC
										melvyl.cdlib.org:210/CDL90
										umnlib.oit.umn.edu:9909/MNCAT
										library.mit.edu:9909/mit01
										atrium.bib.umontreal.ca:210/advance
										bobcat.nyu.edu:210/ADVANCE
										catalogue.nla.gov.au:7090/Voyager
										tegument.nlm.nih.gov:7090/Voyager
										zed.natlib.govt.nz:210/pinz
										z3950.nls.uk:7290/voyager
										zsrv.library.nwu.edu:11090/voyager
										z3950.nb.no:2100/Norper
										aleph.nun.unsw.edu.au:80/NUN
										odin2.bib.sdu.dk:210/Horizon
										janus.uoregon.edu:210/INNOPAC
										library.ox.ac.uk:210/ADVANCE
										cat.libraries.psu.edu:2200/Unicorn
										alpha.bn.org.pl:210/INNOPAC
										catalogue.uquebec.ca:210/uqam
										zinc.rlg.org:210/SAM
										troy.lib.sfu.ca:210/INNOPAC
										carmin.sudoc.abes.fr:210/ABES-Z39-PUBLIC
										z3950.libris.kb.se:210/libr
										db.lib.utexas.edu:210/utcat
										sirsi.library.utoronto.ca:2200/UNICORN
										library.tcd.ie:210/advance
										ucsfcat.library.ucsf.edu:210/INNOPAC
										libdb.lib.upenn.edu:7090/voyager
										library.usc.edu:2200/unicorn
										pulse.uta.edu:7099/pulse
										alpha.lib.uwo.ca:210/INNOPAC
										library.vu.edu.au:210/INNOPAC
										library.usma.army.mil:210/INNOPAC"
								;;
			NATIONAL | N* )				Z39_50_SERVER="amicus.nlc-bnc.ca:210/NL
										z3950.bibsys.no:2100/BIBSYS
										www.bne.es:2210/BIMO
										malvine.bn.pt:2100
										z3950cat.bl.uk:9909/BLAC
										z3950.loc.gov:7090/Voyager
										z3950.copac.ac.uk:2020
										z3950.kb.dk:2100/KGL01
										zeus.statsbiblioteket.dk:210/Horizon
										z3950.gbv.de:20011/gvk
										catalogue.nla.gov.au:7090/Voyager
										tegument.nlm.nih.gov:7090/Voyager
										zed.natlib.govt.nz:210/pinz
										z3950.nls.uk:7290/voyager
										z3950.nb.no:2100/Norper
										alpha.bn.org.pl:210/INNOPAC
										carmin.sudoc.abes.fr:210/ABES-Z39-PUBLIC
										linc.nus.edu.sg:210/innopac
										z3950.libris.kb.se:210/libr
										sirsi.library.utoronto.ca:2200/UNICORN
										library.tcd.ie:210/advance"
								;;
			alberta | ab )				Z39_50_SERVER="dra.library.ualberta.ca:210/Marion" ;;
			amherst | umass )			Z39_50_SERVER="umlibr.library.umass.edu:210/INNOPAC" ;;
			amicus | ca )				Z39_50_SERVER="amicus.nlc-bnc.ca:210/NL" ;;
			anu )					Z39_50_SERVER="library.anu.edu.au:210/INNOPAC" ;;
			be | ulb )				Z39_50_SERVER="www.cible.ulb.ac.be:2200/SirsiDynix" ;;
			bibsys )				Z39_50_SERVER="z3950.bibsys.no:2100/BIBSYS" ;;
			bne | es )				Z39_50_SERVER="www.bne.es:2210/BIMO" ;;
			bnp | pt )				Z39_50_SERVER="malvine.bn.pt:2100"
								Z39_50_TO_BIBTEX="$SHRLIBDIR/sutrs-to-bibtex.awk"
								FORMAT="SUTRS"
								;;
			boulder | co )				Z39_50_SERVER="libraries.colorado.edu:210/INNOPAC" ;;
			british | br )				Z39_50_SERVER="z3950cat.bl.uk:9909/BLAC"
								Z39_50_TO_BIBTEX="$SHRLIBDIR/sutrs-to-bibtex.awk"
								FORMAT="SUTRS"
								URL="http://www.bl.uk/catalogues/z3950copacaccess.html"
								;;
			byu )					Z39_50_SERVER="catalog.lib.byu.edu:2200" ;;
			calgary )				Z39_50_SERVER="clavis.ucalgary.ca:2200/UNICORN" ;;
			caltech )				Z39_50_SERVER="clas.caltech.edu:210/INNOPAC" ;;
			carnegie | cmu )			Z39_50_SERVER="webcat.library.cmu.edu:210/unicorn" ;;
			chicago )				Z39_50_SERVER="ipac.lib.uchicago.edu:210/uofc" ;;
			columbia | cu )				Z39_50_SERVER="clio-db.cc.columbia.edu:7090/Voyager" ;;
			congress | lc | loc )			Z39_50_SERVER="z3950.loc.gov:7090/Voyager" ;;
			copac | uk )				Z39_50_SERVER="z3950.copac.ac.uk:2020" ;;
			cosmos )				Z39_50_SERVER="cosmos.dnlb.dk:9909/Aleph" ;;
			crl )					Z39_50_SERVER="catalog.crl.edu:210/INNOPAC" ;;
			dartmouth )				Z39_50_SERVER="catalog-lib.dartmouth.edu:210/innopac" ;;
			denmark | dk )				Z39_50_SERVER="z3950.kb.dk:2100/KGL01" ;;
			dsb | dsl )				Z39_50_SERVER="zeus.statsbiblioteket.dk:210/Horizon" ;;
			duke )					Z39_50_SERVER="ducatalog.lib.duke.edu:210/MARION" ;;
			edinburgh | ed )			Z39_50_SERVER="catalogue.lib.ed.ac.uk:7090/voyager" ;;
			emory )					Z39_50_SERVER="library.cc.emory.edu:2501/unicorn" ;;
			eu )					Z39_50_SERVER="biblio.iue.it:210/INNOPAC" ;;
			florida | fl )				Z39_50_SERVER="z3950.fcla.edu:210/AM" ;;
			gbv )					Z39_50_SERVER="z3950.gbv.de:20011/gvk" ;;
			gmu )					Z39_50_SERVER="infosparc.gmu.edu:7090/voyager" ;;
			harvard )				Z39_50_SERVER="z3950s://navigator.fas.harvard.edu/boston" ;;
			hopkins | jhu )				Z39_50_SERVER="catalog.lib.jhu.edu:210/horizon" ;;
			home | me )				test -f $CATTOBIBRC &&
									Z39_50_SERVER="`$AWK '{sub(\"#.*$\", \"\")} $1 == \"alias\" && NF >= 3 { for (k = 3; k <= NF; ++k) print $k }' $CATTOBIBRC`"
								test -z "$Z39_50_SERVER" -o ! -f $CATTOBIBRC &&
									error_and_exit "No Z39.50 server aliases found in $CATTOBIBRC"
								;;
			indiana | iu)				Z39_50_SERVER="es33.uits.indiana.edu:2200/unicorn" ;;
			indystate | in )			Z39_50_SERVER="libaix.indstate.edu:210/LUIS" ;;
			kings | kcl )				Z39_50_SERVER="library.kcl.ac.uk:9990/KCL01" ;;
			ku-leuven | ku )			Z39_50_SERVER="cc2.kuleuven.ac.be:3000/xxdefault" ;;
			local )					test -f $SHRLIBRC &&
									Z39_50_SERVER="`$AWK -f $SHRLIBDIR/make-rc-file.awk $SHRLIBRC |
									$AWK '{sub(\"#.*$\", \"\")} $1 == \"alias\" && NF >= 3 { for (k = 3; k <= NF; ++k) print $k }' `"
								test -z "$Z39_50_SERVER" -o ! -f $SHRLIBRC &&
									error_and_exit "No Z39.50 server aliases found in $SHRLIBRC"
								;;
			madrid )				Z39_50_SERVER="biblos.uam.es:2200/unicorn" ;;
			marriott | utah | ut )			Z39_50_SERVER="cos.lib.utah.edu:6060" ;;
			mcgill ) 				Z39_50_SERVER="aleph.mcgill.ca:210/MUSE" ;;
			melbourne )				Z39_50_SERVER="library.unimelb.edu.au:210/INNOPAC" ;;
			melvyl | cal )				Z39_50_SERVER="melvyl.cdlib.org:210/CDL90" ;;
			michigan | mi )				Z39_50_SERVER="z3950.lib.umich.edu:210/miu01_pub" ;;
			minn* | mn )				Z39_50_SERVER="umnlib.oit.umn.edu:9909/MNCAT" ;;
			mit )					Z39_50_SERVER="library.mit.edu:9909/mit01" ;;
			montreal | um )				Z39_50_SERVER="atrium.bib.umontreal.ca:210/advance" ;;
			nbi | bohr )				Z39_50_SERVER="alephext.kb.dk:2100/afg01" ;;
			newyork | ny )				Z39_50_SERVER="bobcat.nyu.edu:210/ADVANCE" ;;
			nla | au )				Z39_50_SERVER="catalogue.nla.gov.au:7090/Voyager" ;;
			nlm | nih )				Z39_50_SERVER="tegument.nlm.nih.gov:7090/Voyager" ;;
			nlnz | nz )				Z39_50_SERVER="zed.natlib.govt.nz:210/pinz" ;;
			nls | scotland )			Z39_50_SERVER="z3950.nls.uk:7290/voyager" ;;
			nlw | cwm | wales )			Z39_50_SERVER="cat.llgc.org.uk:210/DEFAULT" ;;
			northwestern | nw )			Z39_50_SERVER="zsrv.library.nwu.edu:11090/voyager" ;;
			norway | no )				Z39_50_SERVER="z3950.nb.no:2100/Norper" ;;
			nsw | unsw )				Z39_50_SERVER="aleph.nun.unsw.edu.au:80/NUN" ;;
			nus | sg )				Z39_50_SERVER="linc.nus.edu.sg:210/innopac" ;;
			oclc | wc )				Z39_50_SERVER="fsz3950.oclc.org:210/WorldCat"
								SEDOUTPUTFILTER="$SED -f $SHRLIBDIR/WorldCat-to-TeX.sed"
								;;
			odense | sdu )				Z39_50_SERVER="odin2.bib.sdu.dk:210/Horizon" ;;
			oregon | uo )				Z39_50_SERVER="janus.uoregon.edu:210/INNOPAC" ;;
			oxford | ox* )				Z39_50_SERVER="library.ox.ac.uk:210/ADVANCE" ;;
			pennstate | psu | pa )			Z39_50_SERVER="cat.libraries.psu.edu:2200/Unicorn" ;;
			poland | pl )				Z39_50_SERVER="alpha.bn.org.pl:210/INNOPAC"
								SEDINPUTFILTER="$SED -f $SHRLIBDIR/TeX-to-Polish.sed"
								SEDOUTPUTFILTER="$SED -f $SHRLIBDIR/Polish-to-TeX.sed"
								;;
			princeton )				Z39_50_SERVER="catalog.princeton.edu:7090/voyager" ;;
			quebec | uq )				Z39_50_SERVER="catalogue.uquebec.ca:210/uqam" ;;
			rlg )					Z39_50_SERVER="zinc.rlg.org:210/SAM" ;;
			rutgers )				Z39_50_SERVER="www.iris.rutgers.edu:2200/Unicorn" ;;
			sfu )					Z39_50_SERVER="troy.lib.sfu.ca:210/INNOPAC" ;;
			sudoc | abes | fr )			Z39_50_SERVER="carmin.sudoc.abes.fr:210/ABES-Z39-PUBLIC" ;;
			sweden | se )				Z39_50_SERVER="z3950.libris.kb.se:210/libr" ;;
			stanford | su )				Z39_50_SERVER="jenson.stanford.edu:2210/unicorn" ;;
			stockholm )				Z39_50_SERVER="katalog.sub.su.se:7090/Voyager" ;;
			tamu )					Z39_50_SERVER="aubrey.tamu.edu:7090/voyager" ;;
			texas | tx )				Z39_50_SERVER="db.lib.utexas.edu:210/utcat" ;;
			toronto )				Z39_50_SERVER="sirsi.library.utoronto.ca:2200/UNICORN" ;;
			trinity | tcd )				Z39_50_SERVER="library.tcd.ie:210/advance" ;;
			tub | berlin )				Z39_50_SERVER="z3950.kobv.de:9991/u-kbv90" ;;
			tud | darmstadt )			Z39_50_SERVER="tolk.hebis.de:210/hebis" ;;
			tufts )					Z39_50_SERVER="library.tufts.edu:210/INNOPAC" ;;
			ub | bern )				Z39_50_SERVER="opac.nebis.ch:9909/nebis" ;;
			ucc | cork )				Z39_50_SERVER="library.ucc.ie:210/innopac" ;;
			ucd | dublin )				Z39_50_SERVER="udprism01.ucd.ie:2121/prod_talis" ;;
			ucsf )					Z39_50_SERVER="ucsfcat.library.ucsf.edu:210/INNOPAC" ;;
			upenn | penn )				Z39_50_SERVER="libdb.lib.upenn.edu:7090/voyager" ;;
			usc )					Z39_50_SERVER="library.usc.edu:2200/unicorn" ;;
			uta )					Z39_50_SERVER="pulse.uta.edu:7099/pulse" ;;
			uwo )					Z39_50_SERVER="alpha.lib.uwo.ca:210/INNOPAC" ;;
			vanderbilt )				Z39_50_SERVER="acorn.library.vanderbilt.edu:2200/unicorn" ;;
			victoria | vu )				Z39_50_SERVER="library.vu.edu.au:210/INNOPAC" ;;
			westpoint | usma )			Z39_50_SERVER="library.usma.army.mil:210/INNOPAC" ;;
			witwatersrand | wit )			Z39_50_SERVER="innopac.wits.ac.za:210/INNOPAC" ;;
			wustl )					Z39_50_SERVER="catalog.wustl.edu:210/innopac" ;;
			yale )					Z39_50_SERVER="prodorbis.library.yale.edu:7090/voyager" ;;

			### These do not work
			## bell* | z3950.bell-labs.com )	Z39_50_SERVER=z3950.bell-labs.com/books ;;
			## inquirion* | teratext* )		Z39_50_SERVER="z3950.inquirion.com:210" ;;
			## melvyl* | calif* )			Z39_50_SERVER=melvyl.ucop.edu/CAT ;;

			### Default: user-provided address or alias
			### User aliases always override system
			### aliases, so CATTOBIBRC must precede
			### SHRLIBRC on the command line. We preserve
			### the combined and expanded alias file in
			### case --keep-files was specified.
			*)					zalt="`$AWK -f $SHRLIBDIR/make-rc-file.awk $CATTOBIBRC $SHRLIBRC /dev/null 2>/dev/null > $COMBINEDRC ;
									$AWK '{sub(\"#.*$\", \"\")} $1 == \"alias\" && $2 == \"'$z'\" { for (k = 3; k <= NF; ++k) print $k; exit 0}' $COMBINEDRC`" ;
									test -z "$zalt" && zalt="$z"
								Z39_50_SERVER="$zalt"
								;;
			esac
			Z39_50_SERVER_LIST="$Z39_50_SERVER_LIST $Z39_50_SERVER"
		done
		;;

	 -test |  -tes |  -te | \
	--test | --tes | --te )
		TEST_MODE=yes
		;;

	 -title |  -titl |  -tit |  -ti |  -t | \
	--title | --titl | --tit | --ti | --t )
		flags="@attr 1=4"
		;;

	 -USMARC |  -USMAR |  -USMA |  -USM |  -US |  -U | \
	 -usmarc |  -usmar |  -usma |  -usm |  -us |  -u | \
	--USMARC | --USMAR | --USMA | --USM | --US | --U | \
	--usmarc | --usmar | --usma | --usm | --us | --u )
		USMARC=yes
		;;

	 -version |  -versio |  -versi |  -vers |  -ver |  -ve |  -v | \
	--version | --versio | --versi | --vers | --ver | --ve | --v )
		version_and_exit
		;;

	 -volume |  -volum |  -volu |  -vol |  -vo | \
	--volume | --volum | --volu | --vol | --vo )
		flags="@and "
		SERIESVOLUME=1
		;;

	 -year |  -yea |  -ye |  -y | \
	--year | --yea | --ye | --y )
		shift
		YEAR=$1
		;;

	-*)
		error_and_exit unrecognized option "$1"
		;;

	*)
		break
		;;
	esac
	shift
done

test $LOGGER = $CAT && LOGFILE=
test "$Z39_50_SERVER_LIST" = "" && Z39_50_SERVER_LIST="$DEFAULT_Z39_50_SERVER"

printf '%%%%%% -*-BibTeX-*-\n\n'

# mkfifo $FIFO
for Z39_50_SERVER in $Z39_50_SERVER_LIST
do
	for f in "$@"
	do
		case $f in
		[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9Xx] | \
		[-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9Xx] | \
		[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9] | \
		[-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9])
			# Remove hyphens from ISBNs (old 10-digit and new 13-digit)
			f=`echo $f | sed -e s/-//g`
			flags="@attr 1=7"
			;;
		*)
			f=`echo $f | $SEDINPUTFILTER`
			;;
		esac

		## Convert "SERIES <delims> VOLUME" into
		##	flags="@and @attr 1=1066 VOLUME @attr 1=5"
		## and
		##	f="SERIES"

		if test "$SERIESVOLUME" = 1
		then
			series=`echo $f | sed -e 's/ *[^- A-Za-z0-9].*$//'`
			volume=`echo $f | sed -e 's/^.*[^- A-Za-z0-9][^- A-Za-z0-9]* *//'`
			flags="@and @attr 1=1066 $volume @attr 1=5"
			f="$series"
		elif test -n "$YEAR"
		then
			EXTRA="@and @attr 1=30 $YEAR "
		fi

		test $LOGGER = $CAT || echo "%% Searching [$Z39_50_SERVER] for [$f]: flags = [$flags]" 1>&2

		if test "$TEST_MODE" = yes
		then			# test input from stdin instead of from Z39.50 server
			$CAT >$FIFO
		else
			if test $DEBUG = 1
			then
				$CAT 1>&2 <<-EOFILE

					%% BEGIN DEBUG
					$YAZ_CLIENT
					open $Z39_50_SERVER
					format $FORMAT
					find "$EXTRA $flags" "$f"
					show
					...
					show
					close
					%% END DEBUG

EOFILE
			fi
			$EXPECT >$FIFO <<-EOFILE
				spawn $YAZ_CLIENT
				expect "Z>"
				send -- "open $Z39_50_SERVER\r"
				expect "Z>"
				send -- "format $FORMAT\r"
				expect "Z>"
				send -- "find $EXTRA $flags \"$f\"\r"

				# Typical values:
				#	Number of hits: 0
				#	Number of hits: 219
				#	Number of hits: 0, setno 1
				#	Number of hits: 735, setno 1

				expect -re "(Number of hits: *)(\[0-9\]\[0-9\]*)"
				set count \$expect_out(2,string)

				send_tty "\n\nDEBUG: result count = \$count\n\n"

				# we seem to need one more "show" than count, so this test is geq, not gt
				while { \$count >= 0 } {
				    send_tty "DEBUG in while loop: count = \$count  date = [timestamp -format %c]\n"
				    expect "Z>"
				    send -- "show\r"
				    incr count -1
				}

				send -- "close\r"
EOFILE
		fi
		if test $DEBUG = 1
		then
			## Modern FreeBSD, GNU/Linux, Mac OS X, NetBSD, OpenBSD,
			## and Solaris have both /dev/stderr and /dev/fd/2
			## accessible to programs (not just certain shells).
			## IRIX has only /dev/fd/2.  Alas, HP-UX and OSF/1 have
			## neither.  Because test is often a shell builtin, and
			## some shells on those systems fake those devices, we
			## need to avoid the built-in version: TEST is defined
			## above as /bin/test instead.
			if $TEST -w /dev/stderr
			then
				debugfile=/dev/stderr
			elif $TEST -w /dev/fd/2
			then
				debugfile=/dev/fd/2
			else
				debugfile=$TMPDIR/$PROGRAM.debug.$SFX
				$TEST -f $debugfile || echo "%% DEBUG output logged to $debugfile"
			fi
			DEBUGFILTER="$TEE -a $debugfile"
		else
			DEBUGFILTER="$CAT"
		fi
		cat $FIFO |
			eval $DEBUGFILTER |
				eval $SEDOUTPUTFILTER |
					$AWK -f $Z39_50_TO_BIBTEX -v USMARC=$USMARC
	done  > $TMPBIBTEXFILE
	$BIBLABEL $TMPBIBTEXFILE >$LABELFILE

	$CITESUB -f $LABELFILE $TMPBIBTEXFILE 2>/dev/null                                                    |
                                                                                tee /tmp/cattobib.dbg.1.$SFX |
		 $BIBSORT $BIBSORTFLAGS                                                                      |
                                                                                tee /tmp/cattobib.dbg.2.$SFX |
			 $BIBCLEAN $BIBCLEANFLAGS 2>/dev/null                                                |
                                                                                tee /tmp/cattobib.dbg.3.$SFX |
				 $BIBORDER $BIBORDERFLAGS                                                    |
                                                                                tee /tmp/cattobib.dbg.4.$SFX |
					$AWK -f $SHRLIBDIR/iso8859-1-to-TeX.awk                              |
                                                                                tee /tmp/cattobib.dbg.5.$SFX |
						$LOGGER $LOGFILE
done
